@@ -108,9 +108,10 @@ struct print_ops {
 	/* status and error messages */
 	void (*connect_msg)(nvme_ctrl_t c);
 	void (*show_message)(bool error, const char *msg, va_list ap);
-	void (*show_perror)(const char *msg);
+	void (*show_perror)(const char *msg, va_list ap);
 	void (*show_status)(int status);
 	void (*show_error_status)(int status, const char *msg, va_list ap);
+	void (*show_key_value)(const char *key, const char *val, va_list ap);
 
 	nvme_print_flags_t flags;
 };
@@ -300,6 +301,9 @@ const char *nvme_feature_temp_sel_to_string(__u8 sel);
 const char *nvme_feature_temp_type_to_string(__u8 type);
 const char *nvme_feature_to_string(enum nvme_features_id feature);
 const char *nvme_feature_wl_hints_to_string(__u8 wh);
+const char *nvme_feature_perfc_attri_to_string(__u8 attri);
+const char *nvme_feature_perfc_r4karl_to_string(__u8 r4karl);
+const char *nvme_feature_perfc_attrtyp_to_string(__u8 attrtyp);
 const char *nvme_host_metadata_type_to_string(enum nvme_features_id fid, __u8 type);
 const char *nvme_log_to_string(__u8 lid);
 const char *nvme_nss_hw_error_to_string(__u16 error_code);
@@ -322,14 +326,16 @@ const char *nvme_pel_ehai_pit_to_string(enum nvme_pel_ehai_pit pit);
 const char *nvme_ssi_state_to_string(__u8 state);
 const char *nvme_time_scale_to_string(__u8 ts);
 const char *nvme_pls_mode_to_string(__u8 mode);
+const char *nvme_bpwps_to_string(__u8 bpwps);
 
 void nvme_dev_full_path(nvme_ns_t n, char *path, size_t len);
 void nvme_generic_full_path(nvme_ns_t n, char *path, size_t len);
 void nvme_show_message(bool error, const char *msg, ...);
-void nvme_show_perror(const char *msg);
+void nvme_show_perror(const char *msg, ...);
 void nvme_show_error_status(int status, const char *msg, ...);
 void nvme_show_init(void);
 void nvme_show_finish(void);
+void nvme_show_key_value(const char *key, const char *value, ...);
 bool nvme_is_fabrics_reg(int offset);
 bool nvme_is_fabrics_optional_reg(int offset);
 bool nvme_registers_cmbloc_support(__u32 cmbsz);
@@ -352,4 +358,6 @@ void nvme_show_host_discovery_log(struct nvme_host_discover_log *log, nvme_print
 void nvme_show_ave_discovery_log(struct nvme_ave_discover_log *log, nvme_print_flags_t flags);
 void nvme_show_pull_model_ddc_req_log(struct nvme_pull_model_ddc_req_log *log,
 				      nvme_print_flags_t flags);
+
+extern char *alloc_error;
 #endif /* NVME_PRINT_H */
